@@ -59,6 +59,7 @@ typedef enum {
     PREF_TRAY_READ,
     PREF_ADV_NOTIFY_DISCO_OR_VERSION,
     PREF_INTYPE,
+    PREF_INTYPE_CONSOLE,
     PREF_HISTORY,
     PREF_CARBONS,
     PREF_RECEIPTS_SEND,
@@ -67,6 +68,7 @@ typedef enum {
     PREF_OCCUPANTS,
     PREF_OCCUPANTS_SIZE,
     PREF_OCCUPANTS_JID,
+    PREF_OCCUPANTS_OFFLINE,
     PREF_ROSTER,
     PREF_ROSTER_SIZE,
     PREF_ROSTER_OFFLINE,
@@ -165,6 +167,7 @@ typedef enum {
     PREF_STATUSBAR_ROOM,
     PREF_OMEMO_LOG,
     PREF_OMEMO_POLICY,
+    PREF_OMEMO_TRUST_MODE,
     PREF_OCCUPANTS_WRAP,
     PREF_CORRECTION_ALLOW,
     PREF_AVATAR_CMD,
@@ -172,6 +175,8 @@ typedef enum {
     PREF_MAM,
     PREF_URL_OPEN_CMD,
     PREF_URL_SAVE_CMD,
+    PREF_COMPOSE_EDITOR,
+    PREF_SILENCE_NON_ROSTER,
 } preference_t;
 
 typedef struct prof_alias_t
