@@ -40,6 +40,7 @@
 #include <stdlib.h>
 #include <assert.h>
 
+#include "config/accounts.h"
 #include "profanity.h"
 #include "log.h"
 #include "database.h"
@@ -166,8 +167,10 @@ sv_ev_roster_received(void)
     // send initial presence
     resource_presence_t conn_presence = accounts_get_login_presence(account_name);
     char* last_activity_str = accounts_get_last_activity(account_name);
-    if (prefs_get_boolean(PREF_LASTACTIVITY) && last_activity_str) {
+    char* status_message = accounts_get_login_status(account_name);
+    int diff_secs = 0;
 
+    if (prefs_get_boolean(PREF_LASTACTIVITY) && last_activity_str) {
         GTimeVal lasttv;
         GDateTime* nowdt = g_date_time_new_now_utc();
         gboolean res = g_time_val_from_iso8601(last_activity_str, &lasttv);
@@ -175,30 +178,20 @@ sv_ev_roster_received(void)
         if (res) {
             GDateTime* lastdt = g_date_time_new_from_timeval_utc(&lasttv);
             GTimeSpan diff_micros = g_date_time_difference(nowdt, lastdt);
-            int diff_secs = (diff_micros / 1000) / 1000;
-
-            connection_set_presence_msg(NULL);
-            cl_ev_presence_send(conn_presence, diff_secs);
 
+            diff_secs = (diff_micros / 1000) / 1000;
             g_date_time_unref(lastdt);
-        } else {
-            connection_set_presence_msg(NULL);
-            cl_ev_presence_send(conn_presence, 0);
         }
         g_date_time_unref(nowdt);
-    } else {
-        connection_set_presence_msg(NULL);
-        cl_ev_presence_send(conn_presence, 0);
     }
+    connection_set_presence_msg(status_message);
+    cl_ev_presence_send(conn_presence, diff_secs);
 
-    free(last_activity_str);
+    g_free(status_message);
+    g_free(last_activity_str);
 
     const char* fulljid = connection_get_fulljid();
     plugins_on_connect(account_name, fulljid);
-
-#ifdef HAVE_OMEMO
-    omemo_start_sessions();
-#endif
 }
 
 void
@@ -377,8 +370,10 @@ sv_ev_room_message(ProfMessage* message)
 
     if (prefs_do_room_notify(is_current, mucwin->roomjid, mynick, message->from_jid->resourcepart, message->plain, mention, triggers != NULL)) {
         Jid* jidp = jid_create(mucwin->roomjid);
-        notify_room_message(message->from_jid->resourcepart, jidp->localpart, num, message->plain);
-        jid_destroy(jidp);
+        if (jidp) {
+            notify_room_message(message->from_jid->resourcepart, jidp->localpart, num, message->plain);
+            jid_destroy(jidp);
+        }
     }
 
     if (triggers) {
@@ -600,7 +595,9 @@ sv_ev_incoming_message(ProfMessage* message)
     if (message->is_mam) {
         char* mybarejid = connection_get_barejid();
         if (g_strcmp0(mybarejid, message->from_jid->barejid) == 0) {
-            looking_for_jid = message->to_jid->barejid;
+            if (message->to_jid) {
+                looking_for_jid = message->to_jid->barejid;
+            }
         }
         free(mybarejid);
     }
