@@ -34,6 +34,8 @@
  *
  */
 
+#include "config.h"
+
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -244,18 +246,12 @@ _rotate_log_file(void)
             break;
     }
 
-    char* lf = strdup(mainlogfile);
-    char* start = strrchr(lf, '/') + 1;
-    char* end = strstr(start, ".log");
-    *end = '\0';
-
     log_close();
 
     rename(log_file, log_file_new);
 
-    log_init(log_get_filter(), start);
+    log_init(log_get_filter(), NULL);
 
-    free(lf);
     free(log_file_new);
     free(log_file);
     log_info("Log has been rotated");
