@@ -34,6 +34,8 @@
  *
  */
 
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -321,7 +323,14 @@ cons_show_incoming_room_message(const char* const nick, const char* const room,
 
     char* muc_show = prefs_get_string(PREF_CONSOLE_MUC);
 
-    if (g_strcmp0(muc_show, "all") == 0) {
+    // 'mention'
+    if (g_strcmp0(muc_show, "mention") == 0) {
+        if (mention) {
+            win_println(console, THEME_MENTION, "-", "<< room mention: %s in %s (win %d)", nick, room, ui_index);
+            cons_alert(window);
+        }
+        // 'all' or 'first'
+    } else {
         if (mention) {
             win_println(console, THEME_MENTION, "-", "<< room mention: %s in %s (win %d)", nick, room, ui_index);
         } else if (triggers) {
@@ -329,30 +338,15 @@ cons_show_incoming_room_message(const char* const nick, const char* const room,
             win_println(console, THEME_TRIGGER, "-", "<< room trigger %s: %s in %s (win %d)", triggers_str, nick, room, ui_index);
             free(triggers_str);
         } else {
-            win_println(console, THEME_INCOMING, "-", "<< room message: %s in %s (win %d)", nick, room, ui_index);
-        }
-        cons_alert(window);
-
-    } else if (g_strcmp0(muc_show, "first") == 0) {
-        if (mention) {
-            win_println(console, THEME_MENTION, "-", "<< room mention: %s in %s (win %d)", nick, room, ui_index);
-            cons_alert(window);
-        } else if (triggers) {
-            char* triggers_str = _room_triggers_to_string(triggers);
-            win_println(console, THEME_TRIGGER, "-", "<< room trigger %s: %s in %s (win %d)", triggers_str, nick, room, ui_index);
-            free(triggers_str);
-            cons_alert(window);
-        } else if (unread == 0) {
-            win_println(console, THEME_INCOMING, "-", "<< room message: %s (win %d)", room, ui_index);
-            cons_alert(window);
-        }
-    } else if (g_strcmp0(muc_show, "mention") == 0) {
-        if (mention) {
-            win_println(console, THEME_MENTION, "-", "<< room mention: %s in %s (win %d)", nick, room, ui_index);
+            // 'all' or 'first' if its the first message
+            if ((g_strcmp0(muc_show, "all") == 0) || ((g_strcmp0(muc_show, "first") == 0) && (unread == 0))) {
+                win_println(console, THEME_INCOMING, "-", "<< room message: %s in %s (win %d)", nick, room, ui_index);
+            }
             cons_alert(window);
         }
     }
-    g_free(muc_show);
+
+    free(muc_show);
 }
 
 void
@@ -366,10 +360,7 @@ cons_show_incoming_message(const char* const short_from, const int win_index, in
     }
 
     char* chat_show = prefs_get_string(PREF_CONSOLE_CHAT);
-    if (g_strcmp0(chat_show, "all") == 0) {
-        win_println(console, THEME_INCOMING, "-", "<< chat message: %s (win %d)", short_from, ui_index);
-        cons_alert(window);
-    } else if ((g_strcmp0(chat_show, "first") == 0) && unread == 0) {
+    if (g_strcmp0(chat_show, "all") == 0 || ((g_strcmp0(chat_show, "first") == 0) && unread == 0)) {
         win_println(console, THEME_INCOMING, "-", "<< chat message: %s (win %d)", short_from, ui_index);
         cons_alert(window);
     }
@@ -388,15 +379,12 @@ cons_show_incoming_private_message(const char* const nick, const char* const roo
     }
 
     char* priv_show = prefs_get_string(PREF_CONSOLE_PRIVATE);
-    if (g_strcmp0(priv_show, "all") == 0) {
-        win_println(console, THEME_INCOMING, "-", "<< private message: %s in %s (win %d)", nick, room, ui_index);
-        cons_alert(window);
-    } else if ((g_strcmp0(priv_show, "first") == 0) && unread == 0) {
+    if (g_strcmp0(priv_show, "all") == 0 || ((g_strcmp0(priv_show, "first") == 0) && (unread == 0))) {
         win_println(console, THEME_INCOMING, "-", "<< private message: %s in %s (win %d)", nick, room, ui_index);
         cons_alert(window);
     }
 
-    g_free(priv_show);
+    free(priv_show);
 }
 
 void
@@ -518,6 +506,26 @@ cons_show_wins(gboolean unread)
     cons_alert(NULL);
 }
 
+void
+cons_show_wins_attention() {
+    ProfWin* console = wins_get_console();
+    cons_show("");
+    GSList* window_strings = wins_create_summary_attention();
+
+    GSList* curr = window_strings;
+    while (curr) {
+        if (g_strstr_len(curr->data, strlen(curr->data), " unread") > 0) {
+            win_println(console, THEME_CMD_WINS_UNREAD, "-", "%s", curr->data);
+        } else {
+            win_println(console, THEME_DEFAULT, "-", "%s", curr->data);
+        }
+        curr = g_slist_next(curr);
+    }
+    g_slist_free_full(window_strings, free);
+
+    cons_alert(NULL);
+}
+
 void
 cons_show_room_invites(GList* invites)
 {
@@ -736,6 +744,34 @@ cons_show_bookmarks(const GList* list)
     cons_alert(NULL);
 }
 
+void
+cons_show_bookmark(Bookmark* item)
+{
+    cons_show("");
+    if (!item) {
+        cons_show("No such bookmark");
+    } else {
+        cons_show("Bookmark details:");
+        cons_show("Room jid           : %s", item->barejid);
+        if (item->name) {
+            cons_show("name               : %s", item->name);
+        }
+        if (item->nick) {
+            cons_show("nick               : %s", item->nick);
+        }
+        if (item->password) {
+            cons_show("password           : %s", item->password);
+        }
+        if (item->autojoin) {
+            cons_show("autojoin           : ON");
+        } else {
+            cons_show("autojoin           : OFF");
+        }
+    }
+
+    cons_alert(NULL);
+}
+
 void
 cons_show_disco_info(const char* jid, GSList* identities, GSList* features)
 {
@@ -801,6 +837,28 @@ cons_show_disco_items(GSList* items, const char* const jid)
     cons_alert(NULL);
 }
 
+static void _cons_print_contact_information_item(gpointer data, gpointer user_data)
+{
+    cons_show("    %s", (char*)data);
+}
+
+static void _cons_print_contact_information_hashlist_item(gpointer key, gpointer value, gpointer userdata)
+{
+    cons_show("  %s:", (char*)key);
+    g_slist_foreach((GSList*)value, _cons_print_contact_information_item, NULL);
+}
+
+void
+cons_show_disco_contact_information(GHashTable* addresses)
+{
+    if (addresses && g_hash_table_size(addresses) > 0) {
+        cons_show("");
+        cons_show("Server contact information:");
+
+        g_hash_table_foreach(addresses, _cons_print_contact_information_hashlist_item, NULL);
+    }
+}
+
 void
 cons_show_status(const char* const barejid)
 {
@@ -1269,6 +1327,11 @@ cons_occupants_setting(void)
     else
         cons_show("Occupant jids (/occupants)          : hide");
 
+    if (prefs_get_boolean(PREF_OCCUPANTS_OFFLINE))
+        cons_show("Occupants offline (/occupants)      : show");
+    else
+        cons_show("Occupants offline (/occupants)      : hide");
+
     if (prefs_get_boolean(PREF_OCCUPANTS_WRAP))
         cons_show("Occupants wrap (/occupants)         : ON");
     else
@@ -1723,9 +1786,14 @@ void
 cons_intype_setting(void)
 {
     if (prefs_get_boolean(PREF_INTYPE))
-        cons_show("Show typing (/intype)         : ON");
+        cons_show("Show typing in titlebar (/intype titlebar)       : ON");
     else
-        cons_show("Show typing (/intype)         : OFF");
+        cons_show("Show typing in titlebar (/intype titlebar)       : OFF");
+
+    if (prefs_get_boolean(PREF_INTYPE_CONSOLE))
+        cons_show("Show typing in console (/intype console)         : ON");
+    else
+        cons_show("Show typing in console (/intype console)         : OFF");
 }
 
 void
@@ -2084,6 +2152,10 @@ cons_executable_setting(void)
     }
     cons_show("Default '/url save' command (/executable urlsave)                        : %s", urlsave);
     g_free(urlsave);
+
+    gchar* editor = prefs_get_string(PREF_COMPOSE_EDITOR);
+    cons_show("Default '/editor' command (/executable editor)                           : %s", editor);
+    g_free(editor);
 }
 
 void
@@ -2096,6 +2168,26 @@ cons_slashguard_setting(void)
     }
 }
 
+void
+cons_mam_setting(void)
+{
+    if (prefs_get_boolean(PREF_MAM)) {
+        cons_show("Message Archive Management (XEP-0313) (/mam)    : ON");
+    } else {
+        cons_show("Message Archive Management (XEP-0313) (/mam)    : OFF");
+    }
+}
+
+void
+cons_silence_setting(void)
+{
+    if (prefs_get_boolean(PREF_SILENCE_NON_ROSTER)) {
+        cons_show("Block all messages from JIDs that are not in the roster (/silence)    : ON");
+    } else {
+        cons_show("Block all messages from JIDs that are not in the roster (/silence)    : OFF");
+    }
+}
+
 void
 cons_show_connection_prefs(void)
 {
@@ -2312,9 +2404,12 @@ cons_navigation_help(void)
     cons_show("Alt-LEFT, Alt-RIGHT      : Previous/next chat window.");
     cons_show("PAGEUP, PAGEDOWN         : Page the main window.");
     cons_show("Alt-PAGEUP, Alt-PAGEDOWN : Page occupants/roster panel.");
+    cons_show("Alt-a                    : Jump to the next unread window.");
+    cons_show("Alt-v                    : Mark current window with attention flag.");
+    cons_show("Alt-m                    : Switch between marked windows.");
     cons_show("");
-    cons_show("/win <n>     : Focus window n, where n is the window number.");
-    cons_show("/win <name>  : Focus window with name, where name is the recipient, room or window title.");
+    cons_show("/win <n>        : Focus window n, where n is the window number.");
+    cons_show("/win <name>     : Focus window with name, where name is the recipient, room or window title.");
     cons_show("");
     cons_show("See '/help win' for more information.");
 
