@@ -75,7 +75,10 @@ int ui_close_all_wins(void);
 int ui_close_read_wins(void);
 void ui_close_win(int index);
 int ui_win_unread(int index);
-char* ui_ask_password(void);
+gboolean ui_win_has_attention(int index);
+gboolean win_has_attention(ProfWin* window); 
+gboolean win_toggle_attention(ProfWin* window);
+char* ui_ask_password(gboolean confirm);
 char* ui_get_line(void);
 char* ui_ask_pgp_passphrase(const char* hint, int prev_fail);
 void ui_contact_online(char* barejid, Resource* resource, GDateTime* last_activity);
@@ -256,6 +259,7 @@ void cons_show_contacts(GSList* list);
 void cons_show_roster(GSList* list);
 void cons_show_roster_group(const char* const group, GSList* list);
 void cons_show_wins(gboolean unread);
+void cons_show_wins_attention();
 char* cons_get_string(ProfConsoleWin* conswin);
 void cons_show_status(const char* const barejid);
 void cons_show_info(PContact pcontact);
@@ -267,10 +271,12 @@ void cons_show_aliases(GList* aliases);
 void cons_show_login_success(ProfAccount* account, gboolean secured);
 void cons_show_account_list(gchar** accounts);
 void cons_show_room_list(GSList* room, const char* const conference_node);
+void cons_show_bookmark(Bookmark* item);
 void cons_show_bookmarks(const GList* list);
 void cons_show_bookmarks_ignore(gchar** list, gsize len);
 void cons_show_disco_items(GSList* items, const char* const jid);
 void cons_show_disco_info(const char* from, GSList* identities, GSList* features);
+void cons_show_disco_contact_information(GHashTable* addresses);
 void cons_show_room_invite(const char* const invitor, const char* const room, const char* const reason);
 void cons_check_version(gboolean not_available_msg);
 void cons_show_typing(const char* const barejid);
@@ -321,6 +327,8 @@ void cons_os_setting(void);
 void cons_correction_setting(void);
 void cons_executable_setting(void);
 void cons_slashguard_setting(void);
+void cons_mam_setting(void);
+void cons_silence_setting(void);
 void cons_show_contact_online(PContact contact, Resource* resource, GDateTime* last_activity);
 void cons_show_contact_offline(PContact contact, char* resource, char* status);
 void cons_theme_properties(void);
