@@ -33,6 +33,8 @@
  *
  */
 
+#include "config.h"
+
 #include <glib.h>
 
 #include "log.h"
@@ -45,6 +47,9 @@
 #include "omemo/omemo.h"
 
 static int _omemo_receive_devicelist(xmpp_stanza_t* const stanza, void* const userdata);
+static int _omemo_devicelist_publish_result(xmpp_stanza_t* const stanza, void* const userdata);
+static int _omemo_devicelist_configure_submit(xmpp_stanza_t* const stanza, void* const userdata);
+static int _omemo_devicelist_configure_result(xmpp_stanza_t* const stanza, void* const userdata);
 static int _omemo_bundle_publish_result(xmpp_stanza_t* const stanza, void* const userdata);
 static int _omemo_bundle_publish_configure(xmpp_stanza_t* const stanza, void* const userdata);
 static int _omemo_bundle_publish_configure_result(xmpp_stanza_t* const stanza, void* const userdata);
@@ -63,20 +68,44 @@ omemo_devicelist_publish(GList* device_list)
     xmpp_ctx_t* const ctx = connection_get_ctx();
     xmpp_stanza_t* iq = stanza_create_omemo_devicelist_publish(ctx, device_list);
 
+    log_debug("[OMEMO] publish device list");
+
     if (connection_supports(XMPP_FEATURE_PUBSUB_PUBLISH_OPTIONS)) {
         stanza_attach_publish_options(ctx, iq, "pubsub#access_model", "open");
     }
 
+    iq_id_handler_add(xmpp_stanza_get_id(iq), _omemo_devicelist_publish_result, NULL, NULL);
+
     iq_send_stanza(iq);
     xmpp_stanza_release(iq);
 }
 
+void
+omemo_devicelist_configure(void)
+{
+    xmpp_ctx_t* const ctx = connection_get_ctx();
+    char* id = connection_create_stanza_id();
+    Jid* jid = jid_create(connection_get_fulljid());
+
+    xmpp_stanza_t* iq = stanza_create_pubsub_configure_request(ctx, id, jid->barejid, STANZA_NS_OMEMO_DEVICELIST);
+
+    iq_id_handler_add(id, _omemo_devicelist_configure_submit, NULL, NULL);
+
+    iq_send_stanza(iq);
+
+    xmpp_stanza_release(iq);
+    free(id);
+    jid_destroy(jid);
+}
+
 void
 omemo_devicelist_request(const char* const jid)
 {
     xmpp_ctx_t* const ctx = connection_get_ctx();
     char* id = connection_create_stanza_id();
 
+    log_debug("[OMEMO] request device list for jid: %s", jid);
+
     xmpp_stanza_t* iq = stanza_create_omemo_devicelist_request(ctx, id, jid);
     iq_id_handler_add(id, _omemo_receive_devicelist, NULL, NULL);
 
@@ -89,6 +118,7 @@ omemo_devicelist_request(const char* const jid)
 void
 omemo_bundle_publish(gboolean first)
 {
+    log_debug("[OMEMO] publish own OMEMO bundle");
     xmpp_ctx_t* const ctx = connection_get_ctx();
     unsigned char* identity_key = NULL;
     size_t identity_key_length;
@@ -114,7 +144,10 @@ omemo_bundle_publish(gboolean first)
     g_list_free(ids);
 
     if (connection_supports(XMPP_FEATURE_PUBSUB_PUBLISH_OPTIONS)) {
-        stanza_attach_publish_options(ctx, iq, "pubsub#access_model", "open");
+        stanza_attach_publish_options_va(ctx, iq,
+                4, // 2 * number of key-value pairs
+                "pubsub#persist_items", "true",
+                "pubsub#access_model", "open");
     }
 
     iq_id_handler_add(id, _omemo_bundle_publish_result, NULL, GINT_TO_POINTER(first));
@@ -134,6 +167,8 @@ omemo_bundle_request(const char* const jid, uint32_t device_id, ProfIqCallback f
     xmpp_ctx_t* const ctx = connection_get_ctx();
     char* id = connection_create_stanza_id();
 
+    log_debug("[OMEMO] request omemo bundle (jid: %s, device: %d)", jid, device_id);
+
     xmpp_stanza_t* iq = stanza_create_omemo_bundle_request(ctx, id, jid, device_id);
     iq_id_handler_add(id, func, free_func, userdata);
 
@@ -152,6 +187,12 @@ omemo_start_device_session_handle_bundle(xmpp_stanza_t* const stanza, void* cons
     char* from = NULL;
 
     const char* from_attr = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    log_debug("[OMEMO] omemo_start_device_session_handle_bundle: %s", from_attr);
+
+    const char* type = xmpp_stanza_get_type(stanza);
+    if ( g_strcmp0( type, "error") == 0 ) {
+        log_error("[OMEMO] Error to get key for a device from : %s", from_attr);
+    }
 
     if (!from_attr) {
         Jid* jid = jid_create(connection_get_fulljid());
@@ -181,6 +222,7 @@ omemo_start_device_session_handle_bundle(xmpp_stanza_t* const stanza, void* cons
     }
 
     uint32_t device_id = strtoul(++device_id_str, NULL, 10);
+    log_debug("[OMEMO] omemo_start_device_session_handle_bundle: %d", device_id);
 
     xmpp_stanza_t* item = xmpp_stanza_get_child_by_name(items, "item");
     if (!item) {
@@ -417,8 +459,24 @@ out:
 static int
 _omemo_receive_devicelist(xmpp_stanza_t* const stanza, void* const userdata)
 {
-    GList* device_list = NULL;
     const char* from = xmpp_stanza_get_attribute(stanza, STANZA_ATTR_FROM);
+    const char* type = xmpp_stanza_get_type(stanza);
+
+    GList* device_list = NULL;
+
+    if (g_strcmp0(type, STANZA_TYPE_ERROR) == 0) {
+        log_error("[OMEMO] can't get OMEMO device list");
+        xmpp_stanza_t* error = xmpp_stanza_get_child_by_name(stanza, "error");
+        if (!error) {
+            log_error("[OMEMO] missing error element in device list response");
+            return 1;
+        }
+
+        const char* code = xmpp_stanza_get_attribute(error, "code");
+        if (g_strcmp0(code, "404") == 0) {
+            goto out;
+        }
+    }
 
     xmpp_stanza_t* root = NULL;
     xmpp_stanza_t* event = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_PUBSUB_EVENT);
@@ -459,10 +517,10 @@ _omemo_receive_devicelist(xmpp_stanza_t* const stanza, void* const userdata)
     if (current) {
         item = current;
     } else if (first) {
-        log_warning("OMEMO: User %s has a non 'current' device item list: %s.", from, xmpp_stanza_get_id(first));
+        log_warning("[OMEMO] User %s has a non 'current' device item list: %s.", from, xmpp_stanza_get_id(first));
         item = first;
     } else {
-        return 1;
+        goto out;
     }
 
     xmpp_stanza_t* list = xmpp_stanza_get_child_by_ns(item, STANZA_NS_OMEMO);
@@ -480,34 +538,137 @@ _omemo_receive_devicelist(xmpp_stanza_t* const stanza, void* const userdata)
         if (id != NULL) {
             device_list = g_list_append(device_list, GINT_TO_POINTER(strtoul(id, NULL, 10)));
         } else {
-            log_error("OMEMO: received device without ID");
+            log_error("[OMEMO] received device without ID");
         }
     }
 
+out:
     omemo_set_device_list(from, device_list);
 
     return 1;
 }
 
 static int
-_omemo_bundle_publish_result(xmpp_stanza_t* const stanza, void* const userdata)
+_omemo_devicelist_publish_result(xmpp_stanza_t* const stanza, void* const userdata)
+{
+    const char* type = xmpp_stanza_get_type(stanza);
+    if (g_strcmp0(type, STANZA_TYPE_ERROR) == 0) {
+        log_error("[OMEMO] Publishing device list failed");
+
+        xmpp_stanza_t *error = xmpp_stanza_get_child_by_name(stanza, "error");
+        if (!error) {
+            log_error("[OMEMO] Missing error element in device list publication result");
+            return 0;
+        }
+
+        xmpp_stanza_t *pubsub_error = xmpp_stanza_get_child_by_ns(error, STANZA_NS_PUBSUB_ERROR);
+        if (!pubsub_error) {
+            log_error("[OMEMO] Unknown error while publishing our own device list");
+            cons_show_error("Unknown error while publishing our own device list");
+            return 0;
+        }
+
+        if (g_strcmp0(xmpp_stanza_get_name(pubsub_error), "precondition-not-met") == 0) {
+            static gboolean reconfigured = false;
+            if (!reconfigured) {
+                reconfigured = true;
+                cons_show_error("Unable to publish own OMEMO device list, reconfiguring node");
+                log_error("[OMEMO] Unable to publish own OMEMO device list, reconfiguring node");
+                omemo_devicelist_configure();
+            } else {
+                cons_show_error("Unable to publish own OMEMO device list, previous reconfiguration failed. Giving up.");
+                log_error("[OMEMO] Unable to publish own OMEMO device list, previous reconfiguration failed. Giving up.");
+            }
+        }
+    }
+    return 0;
+}
+
+static int
+_omemo_devicelist_configure_submit(xmpp_stanza_t* const stanza, void* const userdata)
+{
+    log_debug("[OMEMO] _omemo_devicelist_configure_submit()");
+
+    xmpp_stanza_t* pubsub = xmpp_stanza_get_child_by_name(stanza, "pubsub");
+    if (!pubsub) {
+        log_error("[OMEMO] The stanza doesn't contain a 'pubsub' child");
+        return 0;
+    }
+    xmpp_stanza_t* configure = xmpp_stanza_get_child_by_name(pubsub, STANZA_NAME_CONFIGURE);
+    if (!configure) {
+        log_error("[OMEMO] The stanza doesn't contain a 'configure' child");
+        return 0;
+    }
+    xmpp_stanza_t* x = xmpp_stanza_get_child_by_name(configure, "x");
+    if (!x) {
+        log_error("[OMEMO] The stanza doesn't contain an 'x' child");
+        return 0;
+    }
+
+    DataForm* form = form_create(x);
+    form_set_value(form,  "pubsub#access_model", "open");
+
+    xmpp_ctx_t* const ctx = connection_get_ctx();
+    Jid* jid = jid_create(connection_get_fulljid());
+    char* id = connection_create_stanza_id();
+    xmpp_stanza_t* iq = stanza_create_pubsub_configure_submit(ctx, id, jid->barejid, STANZA_NS_OMEMO_DEVICELIST, form);
+
+    iq_id_handler_add(id, _omemo_devicelist_configure_result, NULL, NULL);
+
+    iq_send_stanza(iq);
+
+    xmpp_stanza_release(iq);
+    free(id);
+    jid_destroy(jid);
+    return 0;
+}
+
+static int
+_omemo_devicelist_configure_result(xmpp_stanza_t* const stanza, void* const userdata)
 {
     const char* type = xmpp_stanza_get_type(stanza);
 
-    if (g_strcmp0(type, STANZA_TYPE_ERROR) != 0) {
+    if (g_strcmp0(type, STANZA_TYPE_ERROR) == 0) {
+        log_error("[OMEMO] cannot configure device list to an open access model: Result error");
+        return 0;
+    }
+
+    log_debug("[OMEMO] node configured");
+
+    // Try to publish
+    char* barejid = connection_get_barejid();
+    omemo_devicelist_request(barejid);
+
+    free(barejid);
+
+    return 0;
+}
+
+
+static int
+_omemo_bundle_publish_result(xmpp_stanza_t* const stanza, void* const userdata)
+{
+    log_debug("[OMEMO] _omemo_bundle_publish_result()");
+
+    const char* type = xmpp_stanza_get_type(stanza);
+
+    if (g_strcmp0(type, STANZA_TYPE_RESULT) == 0) {
+        log_debug("[OMEMO] bundle published successfully");
         return 0;
     }
 
     if (!GPOINTER_TO_INT(userdata)) {
-        log_error("OMEMO: definitely cannot publish bundle with an open access model");
+        log_error("[OMEMO] definitely cannot publish bundle with an open access model");
         return 0;
     }
 
-    log_info("OMEMO: cannot publish bundle with open access model, trying to configure node");
+    log_debug("[OMEMO] cannot publish bundle with open access model, trying to configure node");
     xmpp_ctx_t* const ctx = connection_get_ctx();
     Jid* jid = jid_create(connection_get_fulljid());
     char* id = connection_create_stanza_id();
     char* node = g_strdup_printf("%s:%d", STANZA_NS_OMEMO_BUNDLES, omemo_device_id());
+    log_debug("[OMEMO] node: %s", node);
+
     xmpp_stanza_t* iq = stanza_create_pubsub_configure_request(ctx, id, jid->barejid, node);
     g_free(node);
 
@@ -524,18 +685,26 @@ _omemo_bundle_publish_result(xmpp_stanza_t* const stanza, void* const userdata)
 static int
 _omemo_bundle_publish_configure(xmpp_stanza_t* const stanza, void* const userdata)
 {
-    /* TODO handle error */
-    xmpp_stanza_t* pubsub = xmpp_stanza_get_child_by_name(stanza, "pubsub");
-    xmpp_stanza_t* configure = xmpp_stanza_get_child_by_name(pubsub, STANZA_NAME_CONFIGURE);
-    xmpp_stanza_t* x = xmpp_stanza_get_child_by_name(configure, "x");
+    log_debug("[OMEMO] _omemo_bundle_publish_configure()");
 
-    DataForm* form = form_create(x);
-    char* tag = g_hash_table_lookup(form->var_to_tag, "pubsub#access_model");
-    if (!tag) {
-        log_info("OMEMO: cannot configure bundle to an open access model");
+    xmpp_stanza_t* pubsub = xmpp_stanza_get_child_by_name(stanza, "pubsub");
+    if (!pubsub) {
+        log_error("[OMEMO] The stanza doesn't contain a 'pubsub' child");
         return 0;
     }
-    form_set_value(form, tag, "open");
+    xmpp_stanza_t* configure = xmpp_stanza_get_child_by_name(pubsub, STANZA_NAME_CONFIGURE);
+    if (!configure) {
+        log_error("[OMEMO] The stanza doesn't contain a 'configure' child");
+        return 0;
+    }
+    xmpp_stanza_t* x = xmpp_stanza_get_child_by_name(configure, "x");
+    if (!x) {
+        log_error("[OMEMO] The stanza doesn't contain an 'x' child");
+        return 0;
+    }
+
+    DataForm* form = form_create(x);
+    form_set_value(form,  "pubsub#access_model", "open");
 
     xmpp_ctx_t* const ctx = connection_get_ctx();
     Jid* jid = jid_create(connection_get_fulljid());
@@ -560,10 +729,13 @@ _omemo_bundle_publish_configure_result(xmpp_stanza_t* const stanza, void* const
     const char* type = xmpp_stanza_get_type(stanza);
 
     if (g_strcmp0(type, STANZA_TYPE_ERROR) == 0) {
-        log_error("OMEMO: cannot configure bundle to an open access model");
+        log_error("[OMEMO] cannot configure bundle to an open access model: Result error");
         return 0;
     }
 
+    log_debug("[OMEMO] node configured");
+
+    // Try to publish again
     omemo_bundle_publish(TRUE);
 
     return 0;
