@@ -33,8 +33,6 @@
  *
  */
 
-#define _GNU_SOURCE 1
-
 #include "config.h"
 
 #ifdef HAVE_GIT_VERSION
@@ -232,9 +230,6 @@ stanza_create_http_upload_request(xmpp_ctx_t* ctx, const char* const id,
     xmpp_stanza_set_name(request, STANZA_NAME_REQUEST);
     xmpp_stanza_set_ns(request, STANZA_NS_HTTP_UPLOAD);
 
-    xmpp_stanza_t* filename = xmpp_stanza_new(ctx);
-    xmpp_stanza_set_name(filename, STANZA_NAME_FILENAME);
-    xmpp_stanza_t* filename_txt = xmpp_stanza_new(ctx);
     char* filename_cpy = strdup(upload->filename);
     // strip spaces from filename (servers don't spaces)
     for (int i = 0; i < strlen(filename_cpy); i++) {
@@ -242,34 +237,16 @@ stanza_create_http_upload_request(xmpp_ctx_t* ctx, const char* const id,
             filename_cpy[i] = '_';
         }
     }
-    xmpp_stanza_set_text(filename_txt, basename(filename_cpy));
+    xmpp_stanza_set_attribute(request, STANZA_ATTR_FILENAME, basename(filename_cpy));
     free(filename_cpy);
-    xmpp_stanza_add_child(filename, filename_txt);
-    xmpp_stanza_add_child(request, filename);
-    xmpp_stanza_release(filename_txt);
-    xmpp_stanza_release(filename);
 
-    xmpp_stanza_t* size = xmpp_stanza_new(ctx);
-    xmpp_stanza_set_name(size, STANZA_NAME_SIZE);
-    xmpp_stanza_t* size_txt = xmpp_stanza_new(ctx);
-    char* filesize = NULL;
-    if (asprintf(&filesize, "%jd", (intmax_t)(upload->filesize)) != -1) {
-        xmpp_stanza_set_text(size_txt, filesize);
-        free(filesize);
+    gchar* filesize = g_strdup_printf("%jd", (intmax_t)(upload->filesize));
+    if (filesize) {
+        xmpp_stanza_set_attribute(request, STANZA_ATTR_SIZE, filesize);
+        g_free(filesize);
     }
-    xmpp_stanza_add_child(size, size_txt);
-    xmpp_stanza_add_child(request, size);
-    xmpp_stanza_release(size_txt);
-    xmpp_stanza_release(size);
 
-    xmpp_stanza_t* content_type = xmpp_stanza_new(ctx);
-    xmpp_stanza_set_name(content_type, STANZA_NAME_CONTENT_TYPE);
-    xmpp_stanza_t* content_type_txt = xmpp_stanza_new(ctx);
-    xmpp_stanza_set_text(content_type_txt, upload->mime_type);
-    xmpp_stanza_add_child(content_type, content_type_txt);
-    xmpp_stanza_add_child(request, content_type);
-    xmpp_stanza_release(content_type_txt);
-    xmpp_stanza_release(content_type);
+    xmpp_stanza_set_attribute(request, STANZA_ATTR_CONTENTTYPE, upload->mime_type);
 
     xmpp_stanza_add_child(iq, request);
     xmpp_stanza_release(request);
@@ -1007,7 +984,7 @@ stanza_create_caps_query_element(xmpp_ctx_t* ctx)
 
     GString* name_str = g_string_new("Profanity ");
     g_string_append(name_str, PACKAGE_VERSION);
-    if (strcmp(PACKAGE_STATUS, "development") == 0) {
+    if (g_strcmp0(PACKAGE_STATUS, "development") == 0) {
 #ifdef HAVE_GIT_VERSION
         g_string_append(name_str, "dev.");
         g_string_append(name_str, PROF_GIT_BRANCH);
@@ -1095,16 +1072,16 @@ stanza_create_caps_sha1_from_query(xmpp_stanza_t* const query)
                 g_string_append(identity_str, name);
             }
             g_string_append(identity_str, "<");
-            identities = g_slist_insert_sorted(identities, g_strdup(identity_str->str), (GCompareFunc)strcmp);
+            identities = g_slist_insert_sorted(identities, g_strdup(identity_str->str), (GCompareFunc)g_strcmp0);
             g_string_free(identity_str, TRUE);
         } else if (g_strcmp0(xmpp_stanza_get_name(child), STANZA_NAME_FEATURE) == 0) {
             const char* feature_str = xmpp_stanza_get_attribute(child, "var");
-            features = g_slist_insert_sorted(features, g_strdup(feature_str), (GCompareFunc)strcmp);
+            features = g_slist_insert_sorted(features, g_strdup(feature_str), (GCompareFunc)g_strcmp0);
         } else if (g_strcmp0(xmpp_stanza_get_name(child), STANZA_NAME_X) == 0) {
             if (g_strcmp0(xmpp_stanza_get_ns(child), STANZA_NS_DATA) == 0) {
                 DataForm* form = form_create(child);
                 char* form_type = form_get_form_type_field(form);
-                form_names = g_slist_insert_sorted(form_names, g_strdup(form_type), (GCompareFunc)strcmp);
+                form_names = g_slist_insert_sorted(form_names, g_strdup(form_type), (GCompareFunc)g_strcmp0);
                 g_hash_table_insert(forms, g_strdup(form_type), form);
             }
         }
@@ -1175,9 +1152,9 @@ stanza_get_child_by_name_and_from(xmpp_stanza_t* const stanza, const char* const
 
     for (child = xmpp_stanza_get_children(stanza); child; child = xmpp_stanza_get_next(child)) {
         child_name = xmpp_stanza_get_name(child);
-        if (child_name && strcmp(name, child_name) == 0) {
+        if (child_name && g_strcmp0(name, child_name) == 0) {
             child_from = xmpp_stanza_get_attribute(child, STANZA_ATTR_FROM);
-            if (child_from && strcmp(from, child_from) == 0) {
+            if (child_from && g_strcmp0(from, child_from) == 0) {
                 break;
             }
         }
@@ -1198,7 +1175,7 @@ _stanza_get_delay_timestamp_xep0203(xmpp_stanza_t* const delay_stanza)
     GTimeVal utc_stamp;
     const char* xmlns = xmpp_stanza_get_attribute(delay_stanza, STANZA_ATTR_XMLNS);
 
-    if (xmlns && (strcmp(xmlns, "urn:xmpp:delay") == 0)) {
+    if (xmlns && (g_strcmp0(xmlns, "urn:xmpp:delay") == 0)) {
         const char* stamp = xmpp_stanza_get_attribute(delay_stanza, STANZA_ATTR_STAMP);
 
         if (stamp && (g_time_val_from_iso8601(stamp, &utc_stamp))) {
@@ -1220,7 +1197,7 @@ _stanza_get_delay_timestamp_xep0091(xmpp_stanza_t* const x_stanza)
     GTimeVal utc_stamp;
     const char* xmlns = xmpp_stanza_get_attribute(x_stanza, STANZA_ATTR_XMLNS);
 
-    if (xmlns && (strcmp(xmlns, "jabber:x:delay") == 0)) {
+    if (xmlns && (g_strcmp0(xmlns, "jabber:x:delay") == 0)) {
         const char* stamp = xmpp_stanza_get_attribute(x_stanza, STANZA_ATTR_STAMP);
         if (stamp && (g_time_val_from_iso8601(stamp, &utc_stamp))) {
 
@@ -1277,7 +1254,7 @@ stanza_get_oldest_delay(xmpp_stanza_t* const stanza)
 
         child_name = xmpp_stanza_get_name(child);
 
-        if (child_name && strcmp(child_name, STANZA_NAME_DELAY) == 0) {
+        if (child_name && g_strcmp0(child_name, STANZA_NAME_DELAY) == 0) {
             GDateTime* tmp = _stanza_get_delay_timestamp_xep0203(child);
 
             if (oldest == NULL) {
@@ -1290,7 +1267,7 @@ stanza_get_oldest_delay(xmpp_stanza_t* const stanza)
             }
         }
 
-        if (child_name && strcmp(child_name, STANZA_NAME_X) == 0) {
+        if (child_name && g_strcmp0(child_name, STANZA_NAME_X) == 0) {
             GDateTime* tmp = _stanza_get_delay_timestamp_xep0091(child);
 
             if (oldest == NULL) {
@@ -1341,7 +1318,7 @@ stanza_is_muc_presence(xmpp_stanza_t* const stanza)
     if (stanza == NULL) {
         return FALSE;
     }
-    if (strcmp(xmpp_stanza_get_name(stanza), STANZA_NAME_PRESENCE) != 0) {
+    if (g_strcmp0(xmpp_stanza_get_name(stanza), STANZA_NAME_PRESENCE) != 0) {
         return FALSE;
     }
 
@@ -1528,11 +1505,8 @@ stanza_get_muc_destroy_alternative_room(xmpp_stanza_t* stanza)
     }
 
     const char* jid = xmpp_stanza_get_attribute(destroy, STANZA_ATTR_JID);
-    if (jid) {
-        return jid;
-    }
 
-    return NULL;
+    return jid;
 }
 
 char*
@@ -1689,7 +1663,7 @@ stanza_get_new_nick(xmpp_stanza_t* const stanza)
     xmpp_stanza_t* x_children = xmpp_stanza_get_children(x);
 
     while (x_children) {
-        if (strcmp(xmpp_stanza_get_name(x_children), STANZA_NAME_ITEM) == 0) {
+        if (g_strcmp0(xmpp_stanza_get_name(x_children), STANZA_NAME_ITEM) == 0) {
             const char* nick = xmpp_stanza_get_attribute(x_children, STANZA_ATTR_NICK);
             if (nick) {
                 return nick;
@@ -1715,7 +1689,7 @@ stanza_get_idle_time(xmpp_stanza_t* const stanza)
         return 0;
     }
 
-    if (strcmp(ns, STANZA_NS_LASTACTIVITY) != 0) {
+    if (g_strcmp0(ns, STANZA_NS_LASTACTIVITY) != 0) {
         return 0;
     }
 
@@ -1777,19 +1751,19 @@ stanza_create_caps_from_query_element(xmpp_stanza_t* query)
             while (field) {
                 formField = field->data;
                 if (formField->values) {
-                    if (strcmp(formField->var, "software") == 0) {
+                    if (g_strcmp0(formField->var, "software") == 0) {
                         if (software == NULL) {
                             software = strdup(formField->values->data);
                         }
-                    } else if (strcmp(formField->var, "software_version") == 0) {
+                    } else if (g_strcmp0(formField->var, "software_version") == 0) {
                         if (software_version == NULL) {
                             software_version = strdup(formField->values->data);
                         }
-                    } else if (strcmp(formField->var, "os") == 0) {
+                    } else if (g_strcmp0(formField->var, "os") == 0) {
                         if (os == NULL) {
                             os = strdup(formField->values->data);
                         }
-                    } else if (strcmp(formField->var, "os_version") == 0) {
+                    } else if (g_strcmp0(formField->var, "os_version") == 0) {
                         if (os_version == NULL) {
                             os_version = strdup(formField->values->data);
                         }
@@ -1940,8 +1914,9 @@ stanza_get_error_message(xmpp_stanza_t* stanza)
     return strdup("unknown");
 }
 
+// Note that the `count' must be 2 * number of key/value pairs
 void
-stanza_attach_publish_options(xmpp_ctx_t* const ctx, xmpp_stanza_t* const iq, const char* const option, const char* const value)
+stanza_attach_publish_options_va(xmpp_ctx_t* const ctx, xmpp_stanza_t* const iq, int count, ...)
 {
     xmpp_stanza_t* publish_options = xmpp_stanza_new(ctx);
     xmpp_stanza_set_name(publish_options, STANZA_NAME_PUBLISH_OPTIONS);
@@ -1964,23 +1939,34 @@ stanza_attach_publish_options(xmpp_ctx_t* const ctx, xmpp_stanza_t* const iq, co
     xmpp_stanza_add_child(form_type, form_type_value);
     xmpp_stanza_add_child(x, form_type);
 
-    xmpp_stanza_t* access_model = xmpp_stanza_new(ctx);
-    xmpp_stanza_set_name(access_model, STANZA_NAME_FIELD);
-    xmpp_stanza_set_attribute(access_model, STANZA_ATTR_VAR, option);
-    xmpp_stanza_t* access_model_value = xmpp_stanza_new(ctx);
-    xmpp_stanza_set_name(access_model_value, STANZA_NAME_VALUE);
-    xmpp_stanza_t* access_model_value_text = xmpp_stanza_new(ctx);
-    xmpp_stanza_set_text(access_model_value_text, value);
-    xmpp_stanza_add_child(access_model_value, access_model_value_text);
-    xmpp_stanza_add_child(access_model, access_model_value);
-    xmpp_stanza_add_child(x, access_model);
-
     xmpp_stanza_t* pubsub = xmpp_stanza_get_child_by_ns(iq, STANZA_NS_PUBSUB);
     xmpp_stanza_add_child(pubsub, publish_options);
 
-    xmpp_stanza_release(access_model_value_text);
-    xmpp_stanza_release(access_model_value);
-    xmpp_stanza_release(access_model);
+    va_list ap;
+    va_start(ap, count);
+    int j;
+    for (j = 0; j < count; j += 2) {
+        const char* const option = va_arg(ap, char* const);
+        const char* const value  = va_arg(ap, char* const);
+
+        xmpp_stanza_t* field = xmpp_stanza_new(ctx);
+        xmpp_stanza_set_name(field, STANZA_NAME_FIELD);
+        xmpp_stanza_set_attribute(field, STANZA_ATTR_VAR, option);
+        xmpp_stanza_t* field_value = xmpp_stanza_new(ctx);
+        xmpp_stanza_set_name(field_value, STANZA_NAME_VALUE);
+        xmpp_stanza_t* field_value_text = xmpp_stanza_new(ctx);
+        xmpp_stanza_set_text(field_value_text, value);
+        xmpp_stanza_add_child(field_value, field_value_text);
+        xmpp_stanza_add_child(field, field_value);
+        xmpp_stanza_add_child(x, field);
+
+        xmpp_stanza_release(field_value_text);
+        xmpp_stanza_release(field_value);
+        xmpp_stanza_release(field);
+    }
+    va_end(ap);
+
+
     xmpp_stanza_release(form_type_value_text);
     xmpp_stanza_release(form_type_value);
     xmpp_stanza_release(form_type);
@@ -1988,6 +1974,13 @@ stanza_attach_publish_options(xmpp_ctx_t* const ctx, xmpp_stanza_t* const iq, co
     xmpp_stanza_release(publish_options);
 }
 
+void
+stanza_attach_publish_options(xmpp_ctx_t* const ctx, xmpp_stanza_t* const iq, const char* const option, const char* const value)
+{
+    stanza_attach_publish_options_va(ctx, iq, 2, option, value);
+}
+
+
 void
 stanza_attach_priority(xmpp_ctx_t* const ctx, xmpp_stanza_t* const presence, const int pri)
 {
@@ -2615,7 +2608,7 @@ stanza_attach_correction(xmpp_ctx_t* ctx, xmpp_stanza_t* stanza, const char* con
 }
 
 xmpp_stanza_t*
-stanza_create_mam_iq(xmpp_ctx_t* ctx, const char* const jid, const char* const startdate)
+stanza_create_mam_iq(xmpp_ctx_t* ctx, const char* const jid, const char* const startdate, const char *const lastid)
 {
     char* id = connection_create_stanza_id();
     xmpp_stanza_t* iq = xmpp_iq_new(ctx, STANZA_TYPE_SET, id);
@@ -2674,6 +2667,23 @@ stanza_create_mam_iq(xmpp_ctx_t* ctx, const char* const jid, const char* const s
 
     xmpp_stanza_add_child(field_start, value_start);
 
+    // 4.3.2 set/rsm
+    xmpp_stanza_t *after, *after_text, *set;
+    if (lastid) {
+        set = xmpp_stanza_new(ctx);
+        xmpp_stanza_set_name(set, STANZA_TYPE_SET);
+        xmpp_stanza_set_ns(set, STANZA_NS_RSM);
+
+        after = xmpp_stanza_new(ctx);
+        xmpp_stanza_set_name(after, STANZA_NAME_AFTER);
+
+        after_text = xmpp_stanza_new(ctx);
+        xmpp_stanza_set_text(after_text, lastid);
+
+        xmpp_stanza_add_child(after, after_text);
+        xmpp_stanza_add_child(set, after);
+    }
+
     // add and release
     xmpp_stanza_add_child(iq, query);
     xmpp_stanza_add_child(query, x);
@@ -2681,6 +2691,14 @@ stanza_create_mam_iq(xmpp_ctx_t* ctx, const char* const jid, const char* const s
     xmpp_stanza_add_child(x, field_with);
     xmpp_stanza_add_child(x, field_start);
 
+    if (lastid) {
+        xmpp_stanza_add_child(query, after);
+
+        xmpp_stanza_release(after_text);
+        xmpp_stanza_release(after);
+        xmpp_stanza_release(set);
+    }
+
     xmpp_stanza_release(mam_text);
     xmpp_stanza_release(with_text);
     xmpp_stanza_release(date_text);
@@ -2695,3 +2713,179 @@ stanza_create_mam_iq(xmpp_ctx_t* ctx, const char* const jid, const char* const s
 
     return iq;
 }
+
+xmpp_stanza_t*
+stanza_change_password(xmpp_ctx_t* ctx, const char* const user, const char* const password)
+{
+    char* id = connection_create_stanza_id();
+    xmpp_stanza_t* iq = xmpp_iq_new(ctx, STANZA_TYPE_SET, id);
+    free(id);
+
+    xmpp_stanza_t* change_password = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(change_password, STANZA_NAME_QUERY);
+    xmpp_stanza_set_ns(change_password, STANZA_NS_REGISTER);
+
+    xmpp_stanza_t* username_st = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(username_st, STANZA_NAME_USERNAME);
+    xmpp_stanza_t* username_text = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_text(username_text, user);
+    xmpp_stanza_add_child(username_st, username_text);
+    xmpp_stanza_release(username_text);
+
+    xmpp_stanza_t* password_st = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(password_st, STANZA_NAME_PASSWORD);
+    xmpp_stanza_t* password_text = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_text(password_text, password);
+    xmpp_stanza_add_child(password_st, password_text);
+    xmpp_stanza_release(password_text);
+
+    xmpp_stanza_add_child(change_password, username_st);
+    xmpp_stanza_release(username_st);
+
+    xmpp_stanza_add_child(change_password, password_st);
+    xmpp_stanza_release(password_st);
+
+    xmpp_stanza_add_child(iq, change_password);
+    xmpp_stanza_release(change_password);
+
+    return iq;
+}
+
+xmpp_stanza_t*
+stanza_request_voice(xmpp_ctx_t* ctx, const char* const room)
+{
+    char* id = connection_create_stanza_id();
+    xmpp_stanza_t* message = xmpp_message_new(ctx, NULL, room, id);
+    free(id);
+
+    xmpp_stanza_t* request_voice_st = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(request_voice_st, STANZA_NAME_X);
+    xmpp_stanza_set_type(request_voice_st, STANZA_TYPE_SUBMIT);
+    xmpp_stanza_set_ns(request_voice_st, STANZA_NS_DATA);
+
+    xmpp_stanza_t* form_type = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(form_type, STANZA_NAME_FIELD);
+    xmpp_stanza_set_attribute(form_type, STANZA_ATTR_VAR, "FORM_TYPE");
+
+    xmpp_stanza_t* value_request = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(value_request, STANZA_NAME_VALUE);
+
+    xmpp_stanza_t* request_text = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_text(request_text, STANZA_NS_VOICEREQUEST);
+
+    xmpp_stanza_add_child(value_request, request_text);
+    xmpp_stanza_release(request_text);
+
+    xmpp_stanza_add_child(form_type, value_request);
+    xmpp_stanza_release(value_request);
+
+    xmpp_stanza_add_child(request_voice_st, form_type);
+    xmpp_stanza_release(form_type);
+
+    xmpp_stanza_t* request_role = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(request_role, STANZA_NAME_FIELD);
+    xmpp_stanza_set_attribute(request_role, STANZA_ATTR_VAR, "muc#role");
+    xmpp_stanza_set_attribute(request_role, STANZA_ATTR_TYPE, "list-single");
+    xmpp_stanza_set_attribute(request_role, STANZA_ATTR_LABEL, "Requested role");
+
+    xmpp_stanza_t* value_role = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(value_role, STANZA_NAME_VALUE);
+
+    xmpp_stanza_t* role_text = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_text(role_text, "participant");
+
+    xmpp_stanza_add_child(value_role, role_text);
+    xmpp_stanza_release(role_text);
+
+    xmpp_stanza_add_child(request_role, value_role);
+    xmpp_stanza_release(value_role);
+
+    xmpp_stanza_add_child(request_voice_st, request_role);
+    xmpp_stanza_release(request_role);
+
+    xmpp_stanza_add_child(message, request_voice_st);
+    xmpp_stanza_release(request_voice_st);
+
+    return message;
+}
+
+xmpp_stanza_t*
+stanza_create_approve_voice(xmpp_ctx_t* ctx, const char* const id, const char* const jid, const char* const node, DataForm* form)
+{
+    char* stid = connection_create_stanza_id();
+    xmpp_stanza_t* message = xmpp_message_new(ctx, NULL, jid, stid);
+    free(stid);
+
+    xmpp_stanza_t* x = form_create_submission(form);
+
+    xmpp_stanza_add_child(message, x);
+    xmpp_stanza_release(x);
+
+    return message;
+}
+
+xmpp_stanza_t*
+stanza_create_muc_register_nick(xmpp_ctx_t* ctx, const char* const id, const char* const jid, const char* const node, DataForm* form)
+{
+    xmpp_stanza_t* iq = xmpp_iq_new(ctx, STANZA_TYPE_SET, id);
+
+    xmpp_stanza_set_to(iq, jid);
+
+    xmpp_stanza_t* x = form_create_submission(form);
+
+    xmpp_stanza_add_child(iq, x);
+    xmpp_stanza_release(x);
+
+    return iq;
+}
+
+static void
+_contact_addresses_list_free(GSList* list)
+{
+    if (list) {
+        g_slist_free_full(list, g_free);
+    }
+}
+
+GHashTable*
+stanza_get_service_contact_addresses(xmpp_ctx_t* ctx, xmpp_stanza_t* stanza)
+{
+    GHashTable* addresses = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)_contact_addresses_list_free);
+
+    xmpp_stanza_t* fields = xmpp_stanza_get_children(stanza);
+    while (fields) {
+        const char* child_name = xmpp_stanza_get_name(fields);
+        const char* child_type = xmpp_stanza_get_type(fields);
+
+        if (g_strcmp0(child_name, STANZA_NAME_FIELD) == 0 && g_strcmp0(child_type, STANZA_TYPE_LIST_MULTI) == 0) {
+            // extract key (eg 'admin-addresses')
+            const char* var = xmpp_stanza_get_attribute(fields, STANZA_ATTR_VAR );
+
+            // extract values (a list of contact addresses eg mailto:xmpp@shakespeare.lit, xmpp:admins@shakespeare.lit)
+            xmpp_stanza_t* values = xmpp_stanza_get_children(fields);
+            GSList* val_list = NULL;
+            while (values) {
+                const char* value_name = xmpp_stanza_get_name(values);
+                if (value_name && (g_strcmp0(value_name, STANZA_NAME_VALUE) == 0)) {
+                    char* value_text = xmpp_stanza_get_text(values);
+                    if (value_text) {
+                        val_list = g_slist_append(val_list, g_strdup(value_text));
+
+                        xmpp_free(ctx, value_text);
+                    }
+                }
+
+                values = xmpp_stanza_get_next(values);
+            }
+
+            // add to list
+            if (g_slist_length(val_list) > 0) {
+                g_hash_table_insert(addresses, g_strdup(var), val_list);
+            }
+        }
+
+        fields = xmpp_stanza_get_next(fields);
+    }
+
+    return addresses;
+}
