@@ -106,6 +106,10 @@
 #define STANZA_NAME_SLOT             "slot"
 #define STANZA_NAME_PUT              "put"
 #define STANZA_NAME_GET              "get"
+#define STANZA_NAME_HEADER           "header"
+#define STANZA_HEADER_AUTHORIZATION  "Authorization"
+#define STANZA_HEADER_COOKIE         "Cookie"
+#define STANZA_HEADER_EXPIRES        "Expires"
 #define STANZA_NAME_URL              "url"
 #define STANZA_NAME_COMMAND          "command"
 #define STANZA_NAME_CONFIGURE        "configure"
@@ -113,6 +117,12 @@
 #define STANZA_NAME_STANZA_ID        "stanza-id"
 #define STANZA_NAME_RESULT           "result"
 #define STANZA_NAME_MINIMIZE         "minimize"
+#define STANZA_NAME_FIN              "fin"
+#define STANZA_NAME_LAST             "last"
+#define STANZA_NAME_AFTER            "after"
+#define STANZA_NAME_USERNAME         "username"
+#define STANZA_NAME_PROPOSE          "propose"
+#define STANZA_NAME_REPORT           "report"
 
 // error conditions
 #define STANZA_NAME_BAD_REQUEST             "bad-request"
@@ -150,6 +160,10 @@
 #define STANZA_TYPE_SET          "set"
 #define STANZA_TYPE_ERROR        "error"
 #define STANZA_TYPE_RESULT       "result"
+#define STANZA_TYPE_SUBMIT       "submit"
+#define STANZA_TYPE_CANCEL       "cancel"
+#define STANZA_TYPE_MODIFY       "modify"
+#define STANZA_TYPE_LIST_MULTI   "list-multi"
 
 #define STANZA_ATTR_TO             "to"
 #define STANZA_ATTR_FROM           "from"
@@ -175,6 +189,10 @@
 #define STANZA_ATTR_STATUS         "status"
 #define STANZA_ATTR_DATE           "date"
 #define STANZA_ATTR_V4_FINGERPRINT "v4-fingerprint"
+#define STANZA_ATTR_FILENAME       "filename"
+#define STANZA_ATTR_SIZE           "size"
+#define STANZA_ATTR_CONTENTTYPE    "content-type"
+#define STANZA_ATTR_LABEL          "label"
 
 #define STANZA_TEXT_AWAY   "away"
 #define STANZA_TEXT_DND    "dnd"
@@ -198,6 +216,7 @@
 #define STANZA_NS_PUBSUB       "http://jabber.org/protocol/pubsub"
 #define STANZA_NS_PUBSUB_OWNER "http://jabber.org/protocol/pubsub#owner"
 #define STANZA_NS_PUBSUB_EVENT "http://jabber.org/protocol/pubsub#event"
+#define STANZA_NS_PUBSUB_ERROR "http://jabber.org/protocol/pubsub#errors"
 #define STANZA_NS_CARBONS      "urn:xmpp:carbons:2"
 #define STANZA_NS_HINTS        "urn:xmpp:hints"
 #define STANZA_NS_FORWARD      "urn:xmpp:forward:0"
@@ -207,7 +226,7 @@
 // XEP-0373: OpenPGP for XMPP
 #define STANZA_NS_OPENPGP_0               "urn:xmpp:openpgp:0"
 #define STANZA_NS_OPENPGP_0_PUBLIC_KEYS   "urn:xmpp:openpgp:0:public-keys"
-#define STANZA_NS_HTTP_UPLOAD             "urn:xmpp:http:upload"
+#define STANZA_NS_HTTP_UPLOAD             "urn:xmpp:http:upload:0"
 #define STANZA_NS_X_OOB                   "jabber:x:oob"
 #define STANZA_NS_BLOCKING                "urn:xmpp:blocking"
 #define STANZA_NS_COMMAND                 "http://jabber.org/protocol/commands"
@@ -220,9 +239,18 @@
 #define STANZA_NS_LAST_MESSAGE_CORRECTION "urn:xmpp:message-correct:0"
 #define STANZA_NS_MAM2                    "urn:xmpp:mam:2"
 #define STANZA_NS_EXT_GAJIM_BOOKMARKS     "xmpp:gajim.org/bookmarks"
+#define STANZA_NS_RSM                     "http://jabber.org/protocol/rsm"
+#define STANZA_NS_REGISTER                "jabber:iq:register"
+#define STANZA_NS_VOICEREQUEST            "http://jabber.org/protocol/muc#request"
+#define STANZA_NS_JINGLE_MESSAGE          "urn:xmpp:jingle-message:0"
+#define STANZA_NS_JINGLE_RTP              "urn:xmpp:jingle:apps:rtp:1"
+#define STANZA_NS_REPORTING               "urn:xmpp:reporting:1"
 
 #define STANZA_DATAFORM_SOFTWARE "urn:xmpp:dataforms:softwareinfo"
 
+#define STANZA_REPORTING_ABUSE "urn:xmpp:reporting:abuse"
+#define STANZA_REPORTING_SPAM  "urn:xmpp:reporting:spam"
+
 typedef struct caps_stanza_t
 {
     char* hash;
@@ -322,7 +350,8 @@ xmpp_stanza_t* stanza_create_room_kick_iq(xmpp_ctx_t* const ctx, const char* con
 xmpp_stanza_t* stanza_create_command_exec_iq(xmpp_ctx_t* ctx, const char* const target, const char* const node);
 xmpp_stanza_t* stanza_create_command_config_submit_iq(xmpp_ctx_t* ctx, const char* const room, const char* const node, const char* const sessionid, DataForm* form);
 
-void stanza_attach_publish_options(xmpp_ctx_t* const ctx, xmpp_stanza_t* const publish, const char* const option, const char* const value);
+void stanza_attach_publish_options_va(xmpp_ctx_t* const ctx, xmpp_stanza_t* const iq, int count, ...);
+void stanza_attach_publish_options(xmpp_ctx_t* const ctx, xmpp_stanza_t* const iq, const char* const option, const char* const value);
 
 xmpp_stanza_t* stanza_create_omemo_devicelist_request(xmpp_ctx_t* ctx, const char* const id, const char* const jid);
 xmpp_stanza_t* stanza_create_omemo_devicelist_subscribe(xmpp_ctx_t* ctx, const char* const jid);
@@ -366,6 +395,8 @@ char* stanza_get_muc_destroy_reason(xmpp_stanza_t* stanza);
 const char* stanza_get_actor(xmpp_stanza_t* stanza);
 char* stanza_get_reason(xmpp_stanza_t* stanza);
 
+GHashTable* stanza_get_service_contact_addresses(xmpp_ctx_t* ctx, xmpp_stanza_t* stanza);
+
 Resource* stanza_resource_from_presence(XMPPPresence* presence);
 XMPPPresence* stanza_parse_presence(xmpp_stanza_t* stanza, int* err);
 void stanza_free_presence(XMPPPresence* presence);
@@ -376,7 +407,10 @@ XMPPCaps* stanza_parse_caps(xmpp_stanza_t* const stanza);
 void stanza_free_caps(XMPPCaps* caps);
 
 xmpp_stanza_t* stanza_create_avatar_retrieve_data_request(xmpp_ctx_t* ctx, const char* stanza_id, const char* const item_id, const char* const jid);
-
-xmpp_stanza_t* stanza_create_mam_iq(xmpp_ctx_t* ctx, const char* const jid, const char* const startdate);
+xmpp_stanza_t* stanza_create_mam_iq(xmpp_ctx_t* ctx, const char* const jid, const char* const startdate, const char* const lastid);
+xmpp_stanza_t* stanza_change_password(xmpp_ctx_t* ctx, const char* const user, const char* const password);
+xmpp_stanza_t* stanza_request_voice(xmpp_ctx_t* ctx, const char* const room);
+xmpp_stanza_t* stanza_create_approve_voice(xmpp_ctx_t* ctx, const char* const id, const char* const jid, const char* const node, DataForm* form);
+xmpp_stanza_t* stanza_create_muc_register_nick(xmpp_ctx_t* ctx, const char* const id, const char* const jid, const char* const node, DataForm* form);
 
 #endif
