@@ -210,8 +210,22 @@ ui_win_unread(int index)
     return 0;
 }
 
+gboolean
+ui_win_has_attention(int index) {
+	return FALSE;
+}
+
+gboolean
+win_has_attention(ProfWin* window){
+	return FALSE;
+}
+
+gboolean
+win_toggle_attention(ProfWin* window){
+	return FALSE;
+}
 char*
-ui_ask_password(void)
+ui_ask_password(gboolean confirm)
 {
     return mock_ptr_type(char*);
 }
@@ -828,6 +842,12 @@ void
 cons_show_wins(gboolean unread)
 {
 }
+
+void 
+cons_show_wins_attention()
+{
+}
+
 void
 cons_show_status(const char* const barejid)
 {
@@ -887,6 +907,11 @@ cons_show_bookmarks(const GList* list)
     check_expected(list);
 }
 
+void
+cons_show_bookmark(Bookmark* item)
+{
+}
+
 void
 cons_show_disco_items(GSList* items, const char* const jid)
 {
@@ -1096,6 +1121,16 @@ void
 cons_slashguard_setting(void)
 {
 }
+void
+cons_mam_setting(void)
+{
+}
+
+void
+cons_silence_setting(void)
+{
+}
+
 void
 cons_show_bookmarks_ignore(gchar** list, gsize len)
 {
